﻿using System.Collections.Generic;
using System.Windows.Media;
using Microsoft.Xna.Framework;
using Swordfish.WPF.Charts;
using Point=System.Windows.Point;
using System.Linq;

namespace Anj.Helpers.Swordfish
{
    public static class SwordfishGraphHelper
    {
        #region Public

        public static void AddLineXY(XYLineChart chart, string label, Color color, IEnumerable<Vector3> points)
        {
            AddLineXY(chart, label, color, points.Select(item => new Point(item.X, item.Y)));
        }

        public static void AddLineXY(XYLineChart chart, string label, Color color, IEnumerable<Point> points)
        {
            ChartPrimitive line = GetLine(points);
            line.Color = color;
            line.Label = label;

            if (chart != null)
                chart.Primitives.Add(line);
        }

        public static void AddLineXY(XYLineChart chart, string label, Color color, IEnumerable<float> values)
        {
            IEnumerable<Point> points = values.Select((val, index) => new Point(index, val));
            AddLineXY(chart, label, color, points);
        }

        #endregion


        public static void AddLineXZ(XYLineChart chart, string label, Color color, IEnumerable<Vector3> points)
        {
            AddLineXY(chart, label, color, points.Select(e => new Point(e.X, e.Z)));
        }

        #region Private helpers

        public static float[] CreateFloatArray(int length, int fillValue)
        {
            var array = new float[length];
            for (int i = 0; i < array.Length; i++)
                array[i] = fillValue;

            return array;
        }

        private static ChartPrimitive GetLine(IEnumerable<Point> points)
        {
            var line = new ChartPrimitive
                           {
                               Filled = false,
                               Dashed = false,
                               ShowInLegend = true,
                               LineThickness = 1.5,
                               HitTest = true
                           };

            foreach (Point point in points)
                line.AddPoint(point);

            return line;
        }

        #endregion


    }
}
