﻿#region Copyright

// A²DS - Autonomous Aerial Drone Simulator
// http://anjdreas.spaces.live.com/
//  
// A Master of Science thesis on autonomous flight at the 
// Norwegian University of Science and Technology (NTNU).
//  
// Copyright © 2009-2010 by Andreas Larsen.  All rights reserved.

#endregion

#region Using

using System;
using Microsoft.Xna.Framework;

#endregion

namespace Anj.Helpers.XNA
{
    public static class MyMathHelper
    {
        /// <summary>
        ///   Returns a linear interpolation from a value in a source interval (from,to) to a value with equal linear progression in a destination interval (from,to).
        ///   Value of 1 in range [0,2] has progress 0.5 and would map to a value of 5 in range [0, 10].
        /// </summary>
        /// <param name="value"></param>
        /// <param name="sourceFrom"></param>
        /// <param name="sourceTo"></param>
        /// <param name="destFrom"></param>
        /// <param name="destTo"></param>
        /// <returns></returns>
        public static float Lerp(float value, float sourceFrom, float sourceTo, float destFrom, float destTo)
        {
            if (sourceFrom == sourceTo) throw new ArgumentException("sourceFrom and sourceTo must be different");
            if (destFrom == destTo) return destTo;

            float invertedValue = (sourceTo - sourceFrom) - (value - sourceFrom) + sourceFrom;

            if (sourceFrom > sourceTo)
                return Lerp(invertedValue, sourceTo, sourceFrom, destFrom, destTo);

            if (destFrom > destTo)
                return Lerp(invertedValue, sourceFrom, sourceTo, destTo, destFrom);

            float rangeProgress = Math.Max(0, Math.Min(1, (value - sourceFrom)/(sourceTo - sourceFrom)));
            return rangeProgress*(destTo - destFrom) + destFrom;
        }

        /// <summary>
        ///   Returns the linear progression of a value from start to end.
        /// </summary>
        /// <param name="value"></param>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        public static float InvLerp(float value, float start, float end)
        {
            if (start >= end) throw new ArgumentException("start must be less than end");
            if (value <= start) return 0;
            if (value >= end) return 1;

            return MathHelper.Clamp((value - start)/(end - start), 0, 1);
        }
    }
}