﻿#region Copyright

// A²DS - Autonomous Aerial Drone Simulator
// http://anjdreas.spaces.live.com/
//  
// A Master of Science thesis on autonomous flight at the 
// Norwegian University of Science and Technology (NTNU).
//  
// Copyright © 2009-2010 by Andreas Larsen.  All rights reserved.

#endregion

#region Using

using System;
using System.Globalization;
using System.Xml.XPath;
using Microsoft.Xna.Framework;

#endregion

namespace Anj.Helpers.XNA
{
    public static class XMLHelper
    {
        private static readonly CultureInfo ParsingCulture;

        static XMLHelper()
        {
            ParsingCulture = CultureInfo.CreateSpecificCulture("en-us");
        }

        /// <summary>
        ///   Convenience extension method that fills in a default namespace as the second attribute.
        /// </summary>
        /// <param name="nav"></param>
        /// <param name="attName"></param>
        /// <returns></returns>
        public static string GetAttribute(this XPathNavigator nav, string attName)
        {
            return nav.GetAttribute(attName, String.Empty);
        }

        public static Vector3 ParseVector3(XPathNavigator node)
        {
            return new Vector3(
                float.Parse(node.GetAttribute("X"), ParsingCulture),
                float.Parse(node.GetAttribute("Y"), ParsingCulture),
                float.Parse(node.GetAttribute("Z"), ParsingCulture));
        }

        public static float[] ParseAttributesFloat(XPathNavigator node, params string[] attNames)
        {
            var result = new float[attNames.Length];
            for (int i = 0; i < attNames.Length; i++)
            {
                string name = attNames[i];
                string attText = node.GetAttribute(name);

                if (!float.TryParse(attText, NumberStyles.Float, ParsingCulture, out result[i]))
                    throw new Exception("Could not parse to float: " + attText);
            }

            return result;
        }

        public static double[] ParseAttributesDouble(XPathNavigator node, params string[] attNames)
        {
            var result = new double[attNames.Length];
            for (int i = 0; i < attNames.Length; i++)
            {
                string name = attNames[i];
                string attText = node.GetAttribute(name);

                if (!double.TryParse(attText, NumberStyles.Float, ParsingCulture, out result[i]))
                    throw new Exception("Could not parse to double: " + attText);
            }

            return result;
        }

        public static int[] ParseAttributesInteger(XPathNavigator node, params string[] attNames)
        {
            var result = new int[attNames.Length];
            for (int i = 0; i < attNames.Length; i++)
            {
                string name = attNames[i];
                string attText = node.GetAttribute(name);

                if (!int.TryParse(attText, NumberStyles.Integer, ParsingCulture, out result[i]))
                    throw new Exception("Could not parse to float: " + attText);
            }

            return result;
        }
    }

//    internal static class TConverter
//    {
//        public static T ChangeType<T>(object value)
//        {
//            return (T)ChangeType(typeof(T), value);
//        }
//        public static object ChangeType(Type t, object value)
//        {
//            TypeConverter tc = TypeDescriptor.GetConverter(t);
//            return tc.ConvertFrom(value);
//        }
//        public static void RegisterTypeConverter<T, TC>() where TC : TypeConverter
//        {
//
//            TypeDescriptor.AddAttributes(typeof(T), new TypeConverterAttribute(typeof(TC)));
//
//        }
//
//    }
}