﻿#region Copyright

// A²DS - Autonomous Aerial Drone Simulator
// http://anjdreas.spaces.live.com/
//  
// A Master of Science thesis on autonomous flight at the 
// Norwegian University of Science and Technology (NTNU).
//  
// Copyright © 2009-2010 by Andreas Larsen.  All rights reserved.

#endregion

#region Using

using System;
using System.Collections.Generic;
using System.IO;
using System.Xml.XPath;
using Anj.Helpers.XNA;
using Anj.XNA.Joysticks;
using Anj.XNA.Joysticks.Wizard;
using Simulator.Utils;

#endregion

namespace Simulator.Parsers
{
    public static class JoystickSetupParserXML
    {
        public static IList<JoystickSetup> GetJoystickSetups(string xmlFilepath)
        {
            var result = new List<JoystickSetup>();

            string xmlText = ParseHelper.GetResourceText(xmlFilepath);
            var reader = new StringReader(xmlText);

            var doc = new XPathDocument(reader);
            XPathNavigator nav = doc.CreateNavigator().SelectSingleNode("/root");
            XPathNodeIterator scenariosIter = nav.Select("JoystickSetup");
            while (scenariosIter.MoveNext())
                result.Add(ParseJoystickSetup(scenariosIter.Current));

            reader.Close();

            return result;
        }

        private static JoystickSetup ParseJoystickSetup(XPathNavigator joystickSetupNode)
        {
            var setup = new JoystickSetup();
            setup.Name = joystickSetupNode.GetAttribute("Name");
            XPathNodeIterator deviceIter = joystickSetupNode.Select("JoystickDevice");

            while (deviceIter.MoveNext())
                setup.Devices.Add(ParseDevice(deviceIter.Current));

            return setup;
        }

        private static JoystickDevice ParseDevice(XPathNavigator deviceNode)
        {
            var device = new JoystickDevice();
            device.Name = deviceNode.GetAttribute("Name");
            XPathNodeIterator axesIter = deviceNode.Select("Axis");
            while (axesIter.MoveNext())
            {
                if (!String.IsNullOrEmpty(axesIter.Current.ToString()))
                    device.Axes.Add(ParseAxis(axesIter.Current));
            }
            return device;
        }

        private static Axis ParseAxis(XPathNavigator axisNode)
        {
            var axis = new Axis();
            axis.Name = ParseJoystickAxis(axisNode);
            axis.Action = ParseJoystickAxisAction(axisNode.Value);
            axis.IsInverted = Boolean.Parse(axisNode.GetAttribute("Inverted"));
            return axis;
        }

        private static JoystickAxis ParseJoystickAxis(XPathNavigator axisNode)
        {
            return (JoystickAxis)Enum.Parse(typeof(JoystickAxis), axisNode.GetAttribute("Name"));
        }

        private static JoystickAxisAction ParseJoystickAxisAction(string actionString)
        {
            return (JoystickAxisAction) Enum.Parse(typeof (JoystickAxisAction), actionString, true);
        }
    }
}